"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const decompress = require('decompress');
const decompressUnzip = require('decompress-unzip');
const https = require('follow-redirects/https');
const log = require('single-line-log').stdout;
const checkSource = (sourcePath) => __awaiter(this, void 0, void 0, function* () {
    let symbolFiles;
    const symbolPath = path.join(sourcePath, 'wrapper', 'symbols');
    const constantsFile = path.join(sourcePath, 'wrapper', 'constants.json');
    try {
        symbolFiles = yield promisify(cb => {
            fs.readdir(symbolPath, cb);
        });
    }
    catch (error) {
        throw new Error(`Could not find symbols in ${symbolPath} from downloaded ZIP file.`);
    }
    try {
        yield promisify(cb => {
            fs.readFile(constantsFile, cb);
        });
    }
    catch (error) {
        throw new Error(`Could not find constants file ${constantsFile} in downloaded ZIP file.`);
    }
    if (!symbolFiles.some(fileName => /.*json/.test(fileName))) {
        throw new Error(`Could not find wrapper files in ${sourcePath}.`);
    }
});
const compareVersionNumbers = (v1, v2) => {
    var v1parts = v1.split('.');
    var v2parts = v2.split('.');
    const isPositiveInteger = (x) => {
        return /^\d+$/.test(x);
    };
    const validateParts = (parts) => {
        for (var i = 0; i < parts.length; ++i) {
            if (!isPositiveInteger(parts[i])) {
                return false;
            }
        }
        return true;
    };
    if (!validateParts(v1parts) || !validateParts(v2parts)) {
        return NaN;
    }
    for (var i = 0; i < v1parts.length; ++i) {
        if (v2parts.length === i) {
            return 1;
        }
        if (v1parts[i] === v2parts[i]) {
            continue;
        }
        if (v1parts[i] > v2parts[i]) {
            return 1;
        }
        return -1;
    }
    if (v1parts.length != v2parts.length) {
        return -1;
    }
    return 0;
};
const httpsGetFileAsync = (url, fileName, showProgress = false) => {
    const MEGABYTE = 0x100000;
    const KILOBYTE = 0x400;
    let startedAt;
    return new Promise((resolve, reject) => {
        https
            .get(url.href, (res) => {
            const { statusCode } = res;
            if (statusCode !== 200) {
                return reject(new Error(`Request to ${url} failed.\nStatus code: ${statusCode}`));
            }
            const file = fs.createWriteStream(fileName);
            const lengthRaw = res.headers['content-length'];
            const usesChunkedEncoding = !lengthRaw;
            const length = usesChunkedEncoding ? 0 : parseInt(lengthRaw, 10);
            let total = usesChunkedEncoding ? 0 : length / MEGABYTE;
            let transferred = 0;
            let elapsed = 0;
            res
                .on('data', chunk => {
                transferred += chunk.length;
                if (usesChunkedEncoding) {
                    total = transferred / MEGABYTE;
                }
                elapsed = (Date.now() - startedAt) / 1000;
                const percent = (100.0 * transferred / length).toFixed(2);
                const speed = transferred / elapsed;
                const speedFormatted = speed < MEGABYTE ? `${~~(speed / KILOBYTE)} kB` : `${(speed / MEGABYTE).toFixed(2)} MB`;
                log(`${usesChunkedEncoding ? '?' : percent + ' %'} of ${total.toFixed(2)} MB (${speedFormatted}/s)`);
            })
                .on('end', () => {
                const seconds = ~~elapsed;
                console.log(` in ${seconds} second${seconds === 1 ? '' : 's'}.`);
                resolve(fileName);
            })
                .pipe(file);
        })
            .on('response', () => (startedAt = Date.now()))
            .on('error', (error) => reject(`Could not download libsodium.js: ${error.message}`));
    });
};
const promisify = (resolver) => {
    return new Promise((resolve, reject) => {
        resolver((err, value) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(value);
            }
        });
    });
};
exports.default = { checkSource, compareVersionNumbers, httpsGetFileAsync, promisify };
